/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.resourceadapter;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.webapp.jonasadmin.deploy.BaseDeployAction;

/**
 * This action is called when the user picks which type of Resource Archive to create.
 *
 * @author Patrick Smith
 */
public class ApplyTemplateCreateResourceAdapterAction extends BaseDeployAction {

    /**
     * The location of the base JOnAS Directory.
     */
    private String jonasBase = JProp.getJonasBase();

    /**
     * The file separator to use for file paths.
     */
    private String fileSeparator =  File.separator;

    /**
     * The name of the directory that stores the rars.
     */
    private String rarDir = "rars";

// --------------------------------------------------------- Public Methods

    /**
     * The action to use when the Struts action is executed.
     * Directs the user to either template depending on which was selected.
     * @param p_Mapping The ActionMapping for the action.
     * @param p_Form The form used in this action.
     * @param p_Request HTTP Request for the action.
     * @param p_Response The HTTP Response.
     * @throws IOException if the there is a problem with the
     * template files, creation of the RARs, or a servlet error.
     * @throws ServletException if the there is a problem with the
     * template files, creation of the RARs, or a servlet error.
     * @return A forward to the next Struts page.
     */
    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Form used
        CreateResourceAdapterForm oForm = (CreateResourceAdapterForm) p_Form;
        m_Session.setAttribute("createResourceAdapterForm", oForm);
        if (oForm.getTemplate().equals("Other")) {
            oForm.setTemplate("Other");
            return (p_Mapping.findForward("Create Other Resource Adapter"));
        } else {
            oForm.setTemplate("JDBC");
            return (p_Mapping.findForward("Create JDBC Resource Adapter"));
        }
    }

// --------------------------------------------------------- Protected Methods

}
