/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 *
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_domain.rules;

import org.apache.commons.digester.Digester;
import org.objectweb.jonas_lib.deployment.rules.JRuleSetBase;

/**
 * This class defines the rules to analyze the domain element
 *
 * @author Adriana Danes
 * @author S. Ali Tokmen
 */
public class DomainRuleSet extends JRuleSetBase {

    /**
     * Construct an object with the prefix "domain"
     */
    public DomainRuleSet() {
        super("domain/");
    }

    /**
     * Add a set of rules to the digester object
     * @param digester Digester instance
     */
    public void addRuleInstances(Digester digester) {
        digester.addCallMethod(prefix + "name", "setName", 0);
        digester.addCallMethod(prefix + "description", "setDescription", 0);
        digester.addCallMethod(prefix + "username", "setUsername", 0);
        digester.addCallMethod(prefix + "password", "setPassword", 0);
        digester.addRuleSet(new ClusterDaemonRuleSet(prefix));
        digester.addRuleSet(new ServerRuleSet(prefix));
        digester.addRuleSet(new ClusterRuleSet(prefix));
    }

}
